// Copyright (C) 2022 RozK
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _RK_ENGINE_MATH_H
#define _RK_ENGINE_MATH_H

#include "types.hpp"
#include <glm/glm.hpp>
#include <glm/ext.hpp>

typedef glm::vec2 rk_vec2;
typedef glm::vec3 rk_vec3;
typedef glm::vec4 rk_vec4;
typedef glm::mat3 rk_mat3;
typedef glm::mat4 rk_mat4;

RK_EXPORT void rk_mat3_rotation(
    rk_mat3 & ret,
    rk_vec3 const & axis,
    float const angle);

RK_EXPORT void rk_mat3_mul_vec3(
    rk_vec3 & ret,
    rk_mat3 const & a,
    rk_vec3 const & b);

RK_EXPORT void rk_mat3_mul_mat3(
    rk_mat3 & ret,
    rk_mat3 const & a,
    rk_mat3 const & b);

RK_EXPORT void rk_mat4_projection(
    rk_mat4 & ret,
    float hfov,
    float ratio,
    float near,
    float far);

RK_EXPORT void rk_mat4_lookat(
    rk_mat4 & ret,
    rk_vec3 const & position,
    rk_vec3 const & lookat);

RK_EXPORT void rk_mat4_mul_vec4(
    rk_vec4 & ret,
    rk_mat4 const & a,
    rk_vec4 const & b);

RK_EXPORT void rk_mat4_mul_mat4(
    rk_mat4 & ret,
    rk_mat4 const & a,
    rk_mat4 const & b);

#endif // _RK_ENGINE_MATH_H
